function cellStyle() {
	this.color = null;
	this.align = null;
	this.bgColor = null;
	this.bgOpacity = null;
	this.bgImageOpacity = null;
	this.font = null;
	this.size = null;
	this.style = null;
}

function GridColumn(key, index) {
	this.key = key;
	this.title = key;
	this.index = index;
	this.width = 100;
	this.height = null;
	this.color = null;
	this.bgColor = null;
	this.bgOpacity = null;
	this.tooltip = null;
	this.font = js_DisplayFont;
	this.size = null;
	this.cellStyle = new cellStyle();
	this.bgImageSrc = null;
	this.visible = true;
	this.onMouseEnter = null;
	this.onMouseExit = null;
	this.onMouseUp = null;
	this.onMouseDown = null;
	this.Object = null;
}

function GridCell(row, column, text, index) {
	this.row = row;
	this.text = text;
	this.column = column;
	this.index = index;
	this.color = null;
	this.bgColor = null;
	this.bgOpacity = null;
	this.bgImageSrc = null;
	this.bgImageOpacity = null;
	this.font = null;
	this.size = null;
	this.style = null;
	this.tooltip = null;
	this.type = GridCellType.Text;
	this.imageSrc = null;
	this.imageWidth = null;
	this.imageHeight = null;
	this.onMouseEnter = null;
	this.onMouseExit = null;
	this.onMouseUp = null;
	this.onMouseDown = null;
	this.Object = null;
}

function GridRow(grid, key, index) {
	this.cells = new Array();
	this.cellObjects = new Array();
	this.key = key.toString();
	this.index = index;
	this.grid = grid;
	this.getCellByColumnKey = function(colKey) {
		for (var i = 0; i < this.cells.length; i++) {
			if (this.cells[i].column.key.toLowerCase() == colKey.toLowerCase())
				return this.cells[i];
		}
		return null;
	}
	this.addCell = function(column, text) {
		var cell = new GridCell(this, column, text, this.cells.length);
		this.cells[this.cells.length] = cell;
		return cell;
	}
}

function Grid() {
	var _frame = null;
	var mainFrame = null;
	this.name = '';
	this.width = 0;
	this.height = 0;
	this.top = 0;
	this.left = 0;
	this.rowHeight = 18;
	this.displayMaxRows = 0;
	this.scrollbarColor = null;
	this.columnBgImageSrc = null;
	this.cellBgImageSrc = null;
	this.rows = new Array();
	this.columns = new Array();
	this.columnObjects = new Array();
	this.clear = function() {
		this.columns.length = 0;
		this.columnObjects.length = 0;
		this.rows.length = 0;
	}
	this.clearRows = function() {
		this.rows.length = 0;
	}
	this.getColumn = function(key) {
		for (var i = 0; i < this.columns.length; i++) {
			if (this.columns[i].key.toLowerCase() == key.toLowerCase())
				return this.columns[i];
		}
		return null;
	}
	this.setRowHeight = function(height) {
		for (var i = 0; i < this.rows.length; i++)
			this.rows[i].height = height;
	}
	this.setColumnColor = function(color) {
		for (var i = 0; i < this.columns.length; i++)
			this.columns[i].color = color;
	}
	this.setColumnBgColor = function(color) {
		for (var i = 0; i < this.columns.length; i++)
			this.columns[i].bgColor = color;
	}
	this.setColumnBgOpacity = function(value) {
		for (var i = 0; i < this.columns.length; i++)
			this.columns[i].bgOpacity = value;
	}

	this.getCell = function(rowIndex, colKey) {
		return this.rows[rowIndex].getCellByColumnKey(colKey);
	}
	this.addColumn = function(key, title) {
		for (var i = 0; i < this.columns.length; i++) {
			if (this.columns[i].key.toLowerCase() == key.toLowerCase())
				throw 'Column Key exists!!!';
		}
		var col = new GridColumn(key, this.columns.length);
		col.title = title;
		this.columns[this.columns.length] = col;
		for (var i = 0; i < this.rows.length; i++) {
			var row = this.rows[i];
			row.addCell(col, '');
		}
		return col;
	}
	this.removeColumn = function(col) {
		var idx = -1;
		if (col.key) {
			idx = col.index;
		} else if (typeof(col) == 'string') {
			for (var i = 0; i < this.columns.length; i++) {
				if (this.columns[i].key.toLowerCase() == col.toLowerCase()) {
					idx = i;
					break;
				}
			}
		} else {
			idx = eval(col);
		}
		if (idx != -1) {
			for (var i = idx; i < this.columns.length - 1; i++) {
				this.columns[i] = this.columns[i + 1];
				this.columns[i].index = i;
			}
			this.columns.length--;
		}
	}

	this.addRow = function(key) {
		for (var i = 0; i < this.rows.length; i++) {
			if (this.rows[i].key.toString().toLowerCase() == key.toString().toLowerCase())
				throw 'Row key exists!!!';
		}
		var row = new GridRow(this, key, this.rows.length);
		for (var i = 0; i < this.columns.length; i++)
			row.addCell(this.columns[i], '');
		this.rows[this.rows.length] = row;
		return row;
	}
	this.removeRow = function(row) {
		var idx = -1;
		if (row.key) {
			idx = row.index;
		} else if (typeof(row) == 'string') {
			for (var i = 0; i < this.rows.length; i++) {
				if (this.rows[i].key.toLowerCase() == row.toLowerCase()) {
					idx = i;
					break;
				}
			}
		} else {
			idx = eval(row);
		}
		if (idx != -1) {
			for (var i = idx; i < this.rows.length - 1; i++) {
				this.rows[i] = this.rows[i + 1];
				this.rows[i].index = i;
			}
			this.rows.length--;
		}
	}

	this.dataBind = function(xml) {
		if (xml == null || xml == '' )
			return;
		if (typeof(xml) == 'string') {
			try {
				var doc = XMLDOM.parse(xml);
			} catch (e) {
				alert(e);
				return false;
			}
		} else {
			doc = xml;
		}
		var root = doc.documentElement;
		this.clear();
		var rows = root.childNodes;
		for (var i = 0; i < rows.length; i++) {
			var cols = rows.item(i).childNodes;
			var row = this.addRow(i);
			for (var j = 0; j < cols.length; j++) {
				var tag = cols.item(j).nodeName;
				var col = this.getColumn(tag);
				if (col == null)
					col = this.addColumn(tag, tag);
				var cell = row.getCellByColumnKey(col.key);
				if (cols.item(j).firstChild != null)
					cell.text = cols.item(j).firstChild.data;

			}
		}
	}
	this.renderControl = function(frame) {
		if (yws_isUndefined(frame)) {
			if (mainFrame == null)
				return;
		} else {
			mainFrame = frame;
		}
		if (_frame != null) {
			_frame.removeFromSuperview();
		}
		var top = 16;
		var left = 4;
		var s = getShadow();
		// Columns
		for (var i = 0; i < this.columns.length; i++) {
			if (!this.columns[i].visible)
				continue;
			if (this.columnBgImageSrc != null || this.columns[i].bgImageSrc != null) {
				var bg = new Image();
			}
			var c = new Text();
			c.data = this.columns[i].title;
			c.width = this.columns[i].width;
			if (this.columns[i].height != null)
				c.height = this.columns[i].height;
			c.hOffset = left + c.width / 2 ;
			c.vOffset = top;
			c.hAlign = 'center';
			c.color = this.columns[i].color;
			if (this.columns[i].bgColor != null)
				c.bgColor = this.columns[i].bgColor;
			if (this.columns[i].bgOpacity != null)
				c.bgOpacity = this.columns[i].bgOpacity;
			if (this.columns[i].font != null)
				c.font = this.columns[i].font;
			if (this.columns[i].size != null)
				c.size = this.columns[i].size;
			if (this.columns[i].tooltip != null)
				c.tooltip = this.columns[i].tooltip;
			c.shadow = s;
			mainFrame.addSubview(c);
			if (this.columnBgImageSrc != null || this.columns[i].bgImageSrc != null) {
				if (this.columns[i].bgImageSrc != null)
					bg.src = this.columns[i].bgImageSrc;
				else
					bg.src = this.columnBgImageSrc;
				bg.hOffset = left;
				bg.vOffset = c.vOffset - c.height + 2;
				bg.width = this.columns[i].width;
				bg.height = c.height + 4;
				this.columnObjects[i] = { Box:bg, Label:c };
				mainFrame.addSubview(bg);
			}
			if (typeof(bg) == 'undefined')
				this.columns[i].Object = { Background:null, Fore:c };
			else
				this.columns[i].Object = { Background:bg, Fore:c };
			left += this.columns[i].width + 1;
		}
		// Rows and cells
		top = c.vOffset + c.height + 5;
		_frame = new Frame();
		with (_frame) {
			hOffset = mainFrame.hOffset;
			vOffset = bg.vOffset + bg.height + 1;
			width = mainFrame.width;
			height = mainFrame.height - bg.height - 10;
		}
		var sbr = new ScrollBar();
		sbr.autoHide = true;
		sbr.setPositionAndSize(_frame.hOffset + _frame.width - 22, 5, 18, mainFrame.height - 10);
		if (this.scrollbarColor == null)
			sbr.thumbColor = '#88AAFF';
		else
			sbr.thumbColor = this.scrollbarColor;
		_frame.vScrollBar = sbr;
		mainFrame.addSubview(sbr);

		top = 0;
		if (this.displayMaxRows != 0 && this.rows.length > this.displayMaxRows)
			var showRows = this.displayMaxRows;
		else
			var showRows = this.rows.length;
		for (var i = 0; i < showRows; i++) {
			left = 4;
			var row = this.rows[i];
			for (var j = 0; j < this.columns.length; j++) {
				if (!this.columns[j].visible)
					continue;

				var cell = row.getCellByColumnKey(this.columns[j].key);
				if (cell == null)
					continue;
				if (this.cellBgImageSrc != null || cell.bgImageSrc != null) {
					var bg = new Image();
					if (cell.bgImageSrc != null)
						bg.src = cell.bgImageSrc;
					else
						bg.src = this.cellBgImageSrc;
					bg.hOffset = left;
					bg.vOffset = top;
					bg.width = this.columns[j].width;
					bg.height = this.rowHeight;
					if (cell.bgImageOpacity != null)
						bg.opacity = cell.bgImageOpacity;
					else if (this.columns[j].cellStyle.bgImageOpacity != null)
						bg.opacity = this.columns[j].cellStyle.bgImageOpacity;
					if (cell.onMouseEnter != null)
						bg.onMouseEnter = cell.onMouseEnter;
					if (cell.onMouseExit != null)
						bg.onMouseExit = cell.onMouseExit;
					if (cell.onMouseDown != null)
						bg.onMouseDown = cell.onMouseDown;
					if (cell.onMouseUp != null)
						bg.onMouseUp = cell.onMouseUp;
					row.cellObjects[i] = { Box:bg, Label:c };
					_frame.addSubview(bg);
				}
				if (cell.type == GridCellType.Text) {
					var c = new Text();
					c.vOffset = top + 13;
					if (cell.font != null)
						c.font = cell.font;
					else if (this.columns[j].cellStyle.font != null)
						c.font = this.columns[j].cellStyle.font;
					if (cell.size != null)
						c.size = cell.size;
					else if (this.columns[j].cellStyle.size != null)
						c.size = this.columns[j].cellStyle.size;
					if (cell.style != null)
						c.style = cell.style;
					else if (this.columns[j].cellStyle.style != null)
						c.style = this.columns[j].cellStyle.style;
					if (cell.text == '')
						c.data = ' ';
					else
						c.data = cell.text;
					c.width = this.columns[j].width;
					if (cell.height != null)
						c.height = cell.height;
					if (cell.color != null)
						c.color = cell.color;
					else if (this.columns[j].cellStyle.color != null)
						c.color = this.columns[j].cellStyle.color;
					switch (this.columns[j].cellStyle.align) {
						case 'left':
							c.hOffset = left;
							break;
						case 'center':
							c.hOffset = left + c.width / 2 ;
							break;
						case 'right':
							c.hOffset = left + c.width;
							break;
						default:
							c.hOffset = left;
							break;
					}
				} else {
					var c = new Image();
					c.src = cell.imageSrc;
					if (cell.imageWidth != null)
						c.width = cell.imageWidth;
					if (cell.imageHeight != null)
						c.height = cell.imageHeight;
					switch (this.columns[j].cellStyle.align) {
						case 'left':
							c.hOffset = left;
							break;
						case 'center':
							c.hOffset = left + this.columns[j].width / 2 ;
							break;
						case 'right':
							c.hOffset = left + this.columns[j].width;
							break;
						default:
							c.hOffset = left;
							break;
					}
					c.vOffset = top + ((this.rowHeight - c.height) / 2);
				}
				if (cell.align != null)
					c.hAlign = cell.align;
				else if (this.columns[j].cellStyle.align != null)
					c.hAlign = this.columns[j].cellStyle.align;
				if (cell.bgColor != null)
					c.bgColor = cell.bgColor;
				else if (this.columns[j].cellStyle.bgColor != null)
					c.bgColor = this.columns[j].cellStyle.bgColor;
				if (cell.bgOpacity != null)
					c.bgOpacity = cell.bgOpacity;
				else if (this.columns[j].cellStyle.bgOpacity != null)
					c.bgOpacity = this.columns[j].cellStyle.bgOpacity;
				if (cell.tooltip != null)
					c.tooltip = cell.tooltip;
				_frame.addSubview(c);

				if (typeof(bg) == 'undefined')
					var b = null;
				else
					b = bg;
				if (cell.type == GridCellType.Text)
					cell.Object = { Background:b, ForeText:c, ForeImage:null };
				else
					cell.Object = { Background:b, ForeText:null, ForeImage:c };
				left += this.columns[j].width + 1;
			}
			top += this.rowHeight + 1;
		}
		mainFrame.addSubview(_frame);
		updateNow();
	}
}
